/****************************************************************************/
/* For the proper operation the PIO, the following functions must be used.  */
/*    PIODA_DriverInit();     <--   initial the driver                        */
/*    PIODA_DriverClose();    <--   close the driver                          */
/****************************************************************************/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "PIODA.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "cspin"
#pragma link "CSPIN"
#pragma resource "*.dfm"

void INT0InitialHigh(WORD wBoardNo, DWORD wBase,
     WORD wCounter0Config, DWORD wCounter0Value);
void INT1InitialHigh(WORD wBoardNo, DWORD wBase,
     WORD wCounter1Config, DWORD wCounter1Value,
     WORD wCounter2Config, DWORD wCounter2Value );

WORD   wTotalBoards, wInitialCode ;
DWORD  dwIntCount;
DWORD  dwServiceCount;
HANDLE h,hEvent,hServiceThread;
DWORD  Tid;
DWORD  wBase;
WORD   wSelectBoard;

//*** This thread is notified by the device driver when a interrupt occurs ***/
DWORD WINAPI ServiceThread(PVOID hEvent)
{
        while (TRUE)
        {
           WaitForSingleObject(hEvent, INFINITE);
           //***** Do some thing in here ****
           //   :
           //   :
           //   :
           dwServiceCount++;
        }

        return 0;
}


TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button1Click(TObject *Sender)
{
     int   wRetVal;

     if ( Button1->Caption == "Stop" )
     {
        Button1->Caption = "Active";
        Button2->Enabled = True;
        return;
     }

     ListBox1->Items->Clear();

     wSelectBoard = (WORD) StrToInt(eSelectBoard->Text);
     if (( wSelectBoard > StrToInt(eTotalBoards->Text) -1))
     {
        ListBox1->Items->Add("Invalid board number, Pls retry!!");
        return ;
     };

     wBase = PIODA_GetBaseAddress( wSelectBoard );
     if ( wBase == 0 )
     {
        ListBox1->Items->Add( "Get Base-Address Error !! " );
        return ;
     }

     //************************************************************
     // enable all DI/DO port
     //************************************************************

     dwIntCount = 0;
     dwServiceCount=0;

     PIODA_IntResetCount();

     ListBox1->Items->Add( "Create Event" );
     hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
     ListBox1->Items->Add( "Create Thread" );
     hServiceThread=CreateThread(0,0x1000,ServiceThread, (PVOID)hEvent, 0, &Tid);

     INT0InitialHigh( wSelectBoard, wBase, 0x36, 20000 );
     //INT1InitialHigh( wSelectBoard, wBase, 0x76, 400, 0xb6, 100 );

     // Initial High and Active Low
     // The Interrupt-Source can be:
     //         0: INT0
     //         1: INT1
     wRetVal=PIODA_IntInstall((WORD)wSelectBoard, &hEvent, 0, PIODA_ActiveLow);
     if ( wRetVal != PIODA_NoError )
     {
        ListBox1->Items->Add("Interrupt Install Error!!");
        ListBox1->Items->Add("Close Thread"     );
        CloseHandle(hServiceThread);
        ListBox1->Items->Add("Close Event" );
        CloseHandle(hEvent);
        return;
     }
     else
        ListBox1->Items->Add("Interrupt Install OK!!");

     Button1->Caption = "Stop";
     Button2->Enabled = False;
     ListBox1->Items->Add("  ");
     ListBox1->Items->Add("Click the button 'Stop' to stop!");
     Application->ProcessMessages();

     for (;;) {
        Sleep(100);
        Application->ProcessMessages();
        if ( Button1->Caption == "Active" )
            break;
     }

     ListBox1->Items->Add( "" );

     // Revmove the Interrupt
     PIODA_IntRemove( );
     ListBox1->Items->Add( "Stop Interrupt!!" );
     ListBox1->Items->Add( "Close Thread"     );
     CloseHandle(hServiceThread);
     ListBox1->Items->Add( "Close Event" );
     CloseHandle(hEvent);

     // How many times the interrupt occur
     wRetVal=PIODA_IntGetCount(&dwIntCount);
     if ( wRetVal != PIODA_NoError )
        ListBox1->Items->Add("Get Interrupt Counter Error!!");
     else
        ListBox1->Items->Add((AnsiString)"Interrupt Counter:" + IntToStr(dwIntCount) );
     ListBox1->Items->Add((AnsiString)"Service function Counter:" + IntToStr(dwServiceCount) );

     ListBox1->Items->Add("end");
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
    wInitialCode = PIODA_DriverInit();
    Button1->Caption = "Active";
    if ( wInitialCode != PIODA_NoError )
    {
        ShowMessage( "Driver initialize error!!") ;
        Button1->Enabled = False;
        return;
    }

    if ( PIODA_SearchCard(&wTotalBoards, PIO_DA) != PIODA_NoError )
    {
        ShowMessage( "Search Board error !!" );
        Button1->Enabled = False;
    }
    else
        Button1->Enabled = True;

    eSelectBoard->Text  = "0";
    eTotalBoards->Text  = IntToStr(wTotalBoards);
    if ( wTotalBoards < 2 ) eSelectBoard->Enabled = False;
    eSelectBoard->MaxValue = wTotalBoards - 1;
    eSelectBoard->MinValue = 0;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
      PIODA_DriverClose();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button2Click(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------

// ******************************************************************
// Initialize the INT0 Interrupt to High
// this will uses the Counter0 to trigger the interrupt.
//
// Parameters :
//   wBoardNo        :[in] Board Number, Range is 0 to 7.
//   wBase           :[in] The base-address of the board
//   wCounter0Config :[in] The configuration code of Counter0
//   wCounter0Value  :[in] 0 to &hFFFF, the value is used to set the Counter0
//                         Only the low WORD (16-bits) is valid.
// ******************************************************************
void INT0InitialHigh(WORD wBoardNo, DWORD wBase,
     WORD wCounter0Config, DWORD wCounter0Value)
{
    // Cout0 as 4Mhz/ wCounter0Value
    // Counter 0
    PIODA_SetCounter(wBoardNo, 0, wCounter0Config, wCounter0Value);

    // wait for Cout0 to high
    for( ;  ;  )
    {
       if( (PIODA_InputByte(wBase+0x07)&1) )
           break;
    }
}

// ******************************************************************
// Initialize the INT1 Interrupt to High,
// this will uses the Counter1 and Counter2 to trigger the interrupt.
//
// Parameters :
//   wBoardNo        :[in] Board Number, Range is 0 to 7.
//   wBase           :[in] The base-address of the board
//   wCounter1Config :[in] The configuration code of Counter1
//   wCounter1Value  :[in] 0 to &hFFFF, the value is used to set the Counter1
//                         Only the low WORD (16-bits) is valid.
//   wCounter2Config :[in] The configuration code of Counter2
//   wCounter2Value  :[in] 0 to &hFFFF, the value is used to set the Counter2
//                         Only the low WORD (16-bits) is valid.
// ******************************************************************
void INT1InitialHigh(WORD wBoardNo, DWORD wBase,
     WORD wCounter1Config, DWORD wCounter1Value,
     WORD wCounter2Config, DWORD wCounter2Value )
{
    // Cout2 as 4Mhz/( wCounter1Value * wCounter2Value)
    // Counter 1
    PIODA_SetCounter(wBoardNo, 1, wCounter1Config, wCounter1Value);
    // Counter 2
    PIODA_SetCounter(wBoardNo, 2, wCounter2Config, wCounter2Value);

    // wait for Cout2 to high
    for( ;  ;  )
    {
       if( (PIODA_InputByte(wBase+0x07)&3) )
           break;
    }
}


