using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PIODA_Ns;//PIODA nameSpace

namespace Int
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        uint wBase;
	    ushort wTotalBoards;
	    ushort wInitialCode;
	    uint dwIntCount;
        int hEvent;

        [System.Runtime.InteropServices.DllImport("kernel32")]
        static extern int SleepEx(int dwMilliseconds, int bAlertable);
        [System.Runtime.InteropServices.DllImport("kernel32")]
        static extern int WaitForSingleObject(int hHandle, int dwMilliseconds);
        [System.Runtime.InteropServices.DllImport("kernel32", EntryPoint = "CreateEventA")]
        static extern int CreateEvent(int lpEventAttributes, int bManualReset, int bInitialState, int lpName);
        [System.Runtime.InteropServices.DllImport("kernel32")]
        static extern int CloseHandle(int hObject);
        

        
	
	
	//' ******************************************************************
	//' Initialize the INT0 Interrupt to High
	//' this will uses the Counter0 to trigger the interrupt.
	//'
	//' Parameters :
	//'   wBoardNo        :[in] Board Number, Range is 0 to 7.
	//'   wCounter0Config :[in] The configuration code of Counter0
	//'   wCounter0Value  :[in] 0 to &hFFFF, the value is used to set the Counter0
	//'                         Only the low Word (16-bits) is valid.
	//' ******************************************************************
        public void INT0InitialHigh(ushort wBoardNo, uint wBase, ushort wCounter0Config, uint wCounter0Value)
        {
            // Cout0 as 4Mhz/ wCounter0Value
		    // Counter 0
            PIODA.SetCounter(wBoardNo, 0, wCounter0Config, wCounter0Value);

            //' wait for Cout0 to high
            while ((true))
            {
                if (((PIODA.InputByte(wBase + 7) & 1) != 0))
                {
                    return;
                }
            }
        }

        
	//' ******************************************************************
	//' Initialize the INT1 Interrupt to High,
	//' this will uses the Counter1 and Counter2 to trigger the interrupt.
	//'
	//' Parameters :
	//'   wBoardNo        :[in] Board Number, Range is 0 to 7.
	//'   wCounter1Config :[in] The configuration code of Counter1
	//'   wCounter1Value  :[in] 0 to &hFFFF, the value is used to set the Counter1
	//'                         Only the low Word (16-bits) is valid.
	//'   wCounter2Config :[in] The configuration code of Counter2
	//'   wCounter2Value  :[in] 0 to &hFFFF, the value is used to set the Counter2
	//'                         Only the low Word (16-bits) is valid.
	//' ******************************************************************
        public void INT1InitialHigh(ushort wBoardNo,uint wBase, ushort wCounter1Config, uint wCounter1Value, ushort wCounter2Config, uint wCounter2Value)
        {
            //' Cout2 as 4Mhz/( wCounter1Value * wCounter2Value)
		    //' Counter 1
            PIODA.SetCounter(wBoardNo, 1, wCounter1Config, wCounter1Value);
            //'Counter 2
            PIODA.SetCounter(wBoardNo, 2, wCounter2Config, wCounter2Value);
            //' wait for Cout2 to high
            while ((true))
            {
                if (((PIODA.InputByte(wBase + 7) & 3) != 0))
                {
                    return;
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
          
        //'********************************************************************
        //'* NOTICE: call PIODA_DriverInit() to initialize the driver.        *
        //'* Initial the device driver, and return the board number in the PC *
        //'********************************************************************

            wInitialCode = PIODA.DriverInit();
            btnActive.Text = "Active";
            if (wInitialCode != PIODA.NoError)
            {
                MessageBox.Show ("Driver initialize error!!!", "PIODA Card Error");
                btnActive.Enabled = false;
                return;
            }
            if (PIODA.SearchCard(out wTotalBoards, PIODA.ID) != PIODA.NoError)
            {
                MessageBox.Show("Search Card Error!!", "PIODA Card Error");
                btnActive.Enabled = false;
            }
            else
            {
                btnActive.Enabled = true;
            }
            eTotalBoards.Text = Convert.ToString(wTotalBoards);
            nudActive.Minimum = 0;
            nudActive.Maximum = wTotalBoards - 1;
        }

        private void btnActive_Click(object sender, EventArgs e)
        {
            ushort wRetVal;
            ushort wSelectBoard;
            uint lRtn;
            if (btnActive.Text == "Stop")
            {
                btnActive.Text = "Active";
                btnExit.Enabled = true;
                return;
            }
            List1.Items.Clear();
            wSelectBoard =(ushort)nudActive.Value;
            if (wSelectBoard > Convert.ToInt16(eTotalBoards.Text) - 1 | wSelectBoard < 0)
            {
                List1.Items.Add("Invalid board number, Pls retry!!");
                return;
            }
            wBase = PIODA.GetBaseAddress(wSelectBoard);
            if (wBase == 0)
            {
                List1.Items.Add("Get Base-Address Error !!");
                return;
            }
            PIODA.IntResetCount();
            dwIntCount = 0;
            hEvent = CreateEvent(0, 0, 0, 0);
            if (hEvent == 0)
            {
                List1.Items.Add("Create Event Error!!");
                return;
            }
            else
            {
                List1.Items.Add("Create Event OK!!");
            }

             //'INT1InitialHigh  wSelectBoard, wBase, &h76, 400, &hb6, 100
             // Initial High and Active Low
             // The Interrupt-Source can be:
             //         0: INT0
             //         1: INT1
            INT0InitialHigh(wSelectBoard, wBase, 54, 20000);
            wRetVal = PIODA.IntInstall(wSelectBoard,out hEvent, 0, PIODA.ActiveLow);
            if (wRetVal != PIODA.NoError)
            {
                List1.Items.Add("Interrupt Install Error!!");
                List1.Items.Add("Close Event");
                CloseHandle(hEvent);
                return;
            }
            else
            {
                List1.Items.Add("Interrupt Install OK!!");
            }
            btnActive.Text = "Stop";
            btnExit.Enabled = false;
            List1.Items.Add(" ");
            List1.Items.Add("Click the button 'Stop' to stop!");
            System.Windows.Forms.Application.DoEvents();
            int j;
            j = 0;
            while ((true))
            {
                List1.Items.Add("(#" + System.Convert.ToString(j) + ") Wait Event .....");
                System.Windows.Forms.Application.DoEvents();
                lRtn =(uint)WaitForSingleObject(hEvent, 0xffff);
                List1.Items.Add("(#" + System.Convert.ToString(j) + ") Got Event !!");
                System.Windows.Forms.Application.DoEvents();
                List1.Items.Add(" ");
                List1.SelectedIndex = List1.Items.Count - 1;
                System.Windows.Forms.Application.DoEvents();
                if (btnActive.Text == "Active")
                {
                    goto exitDoLoopStatement0;
                }
                j = j + 1;
            }
        exitDoLoopStatement0: ;
            List1.Items.Add(" ");
            PIODA.IntRemove();
            List1.Items.Add("Stop Interrupt!!");
            CloseHandle(hEvent);
            wRetVal = PIODA.IntGetCount(out dwIntCount);
            if (wRetVal != PIODA.NoError)
            {
                List1.Items.Add("Get Interrupt Counter Error!!");
            }
            else
            {
                List1.Items.Add("Interrupt Counter:" + Convert.ToString(dwIntCount));
            }
            List1.Items.Add("end");
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            PIODA.DriverClose();
            System.Environment.Exit(0);

        }


    }
}