//****************************************************************************/
//* For the proper operation the PIODA, the following functions must be used.  */
//*    PIODA_DriverInit();     <--   initial the driver                        */
//*    PIODA_DriverClose();    <--   close the driver                          */
//****************************************************************************/

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    eTotalBoards: TEdit;
    eSelectBoard: TEdit;
    Label2: TLabel;
    UpDown1: TUpDown;
    Button1: TButton;
    Button2: TButton;
    ListBox1: TListBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  function ServiceThread(hEvent:THANDLE):DWORD; StdCall;
  procedure INT0InitialHigh(wBoardNo:Word; wBase:LongInt;
            wCounter0Config:Word; wCounter0Value:LongInt);
  procedure INT1InitialHigh(wBoardNo:Word; wBase:LongInt;
            wCounter1Config:Word; wCounter1Value:LongInt;
            wCounter2Config:Word; wCounter2Value:LongInt);

var
  Form1: TForm1;

implementation

uses PIODA;

{$R *.DFM}

Var
   wTotalBoards, wInitialCode : Word;
   dwServiceCount :DWORD ;
   dwIntCount     :LongInt ;
   hEvent,hServiceThread :THANDLE ;
   Tid :DWORD;

//*** This thread is notified by the device driver when a interrupt occurs ***/
function ServiceThread(hEvent:THANDLE):DWORD; StdCall;
begin
        result := 0;

        while (TRUE) do
        begin
           WaitForSingleObject(hEvent, INFINITE);
           //***** Do some thing in here ****
           //   :
           //   :
           //   :
           inc( dwServiceCount );
        end;
end;


procedure TForm1.Button1Click(Sender: TObject);
Var
     wRetVal      : Word;
     wSelectBoard : Word;
     wBase        : LongInt;
begin
     if ( Button1.Caption = 'Stop' ) then
     begin
        Button1.Caption := 'Active';
        Button2.Enabled := True;
        exit;
     end;

     ListBox1.Items.Clear();

     wSelectBoard := StrToInt(eSelectBoard.Text);
     if ( wSelectBoard > StrToInt(eTotalBoards.Text) -1)
     or ( wSelectBoard < 0 ) then
     Begin
         ListBox1.Items.Add('Invalid board number, Pls retry!!');
         Exit;
     end;

     wBase := PIODA_GetBaseAddress( wSelectBoard );
     if wBase = 0 then
     Begin
         ListBox1.Items.Add('Error in get the base-address!!');
         Exit;
     end;

     dwIntCount := 0;
     dwServiceCount:=0;

     PIODA_IntResetCount();
     ListBox1.Items.Add( 'Create Event'  );
     hEvent := CreateEvent(Nil, FALSE, FALSE, Nil);
     ListBox1.Items.Add( 'Create Thread' );
     hServiceThread := CreateThread(Nil,$1000,@ServiceThread, Pointer(hEvent), 0, Tid);

     INT0InitialHigh( wSelectBoard, wBase, $36, 20000 );
     // INT1InitialHigh( wSelectBoard, wBase, $76, 400, $b6, 100 );

     // Initial High and Active Low
     // The Interrupt-Source can be:
     //         0: INT0
     //         1: INT1
     wRetVal := PIODA_IntInstall( wSelectBoard, LongInt(hEvent), 0, PIODA_ActiveLow);
     if wRetVal <> PIODA_NoError then
     begin
        ListBox1.Items.Add('Interrupt Install Error!!');
        ListBox1.Items.Add('Close Thread');
        CloseHandle(hServiceThread);
        ListBox1.Items.Add('Close Event');
        CloseHandle(hEvent);
        exit;
     end
     else
        ListBox1.Items.Add('Interrupt Install OK!!');

     Button1.Caption := 'Stop';
     Button2.Enabled := False;
     ListBox1.Items.Add('  ');
     ListBox1.Items.Add('Click the button "Stop" to stop!');
     Application.ProcessMessages();

     while (true) do
     begin
        Sleep(100);
        Application.ProcessMessages();
        if Button1.Caption = 'Active' then
           break;
     end;

     ListBox1.Items.Add( '' );

     // Revmove the Interrupt
     PIODA_IntRemove();
     ListBox1.Items.Add( 'Stop Interrupt!!' );
     ListBox1.Items.Add( 'Close Thread'     );
     CloseHandle(hServiceThread);
     ListBox1.Items.Add( 'Close Event' );
     CloseHandle(hEvent);

     // How many times the interrupt occur
     wRetVal := PIODA_IntGetCount(dwIntCount);
     if wRetVal <> PIODA_NoError then
        ListBox1.Items.Add('Get Interrupt Counter Error!!')
     else
        ListBox1.Items.Add('Interrupt Counter:' + IntToStr(dwIntCount) );
     ListBox1.Items.Add('Service function Counter:' + IntToStr(dwServiceCount) );

     ListBox1.Items.Add('end');
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
     Close();
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    wInitialCode := PIODA_DriverInit;
    Button1.Caption := 'Active';
    If wInitialCode <> PIODA_NoError Then
    begin
        ShowMessage('Driver Initialize error!!!');
        Button1.Enabled := False;
    end;

    if PIODA_SearchCard(wTotalBoards, PIO_DA) <> PIODA_NoError then
    begin
        ShowMessage('Search Card Error!!!');
        Button1.Enabled := False;
    end
    else
        Button1.Enabled := True;

    eSelectBoard.Text := '0';
    eTotalBoards.Text := IntToStr(wTotalBoards);
    Updown1.Max := wTotalBoards - 1;
    Updown1.Min := 0;

end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
        PIODA_DriverClose();
end;

// ******************************************************************
// Initialize the INT0 Interrupt to High
// this will uses the Counter0 to trigger the interrupt.
//
// Parameters :
//   wBoardNo        :[in] Board Number, Range is 0 to 7.
//   wBase           :[in] The base-address of the board
//   wCounter0Config :[in] The configuration code of Counter0
//   wCounter0Value  :[in] 0 to &hFFFF, the value is used to set the Counter0
//                         Only the low Word (16-bits) is valid.
// ******************************************************************
procedure INT0InitialHigh(wBoardNo:Word; wBase:LongInt;
          wCounter0Config:Word; wCounter0Value:LongInt);
begin
   // Cout2 as 4Mhz/ wCounter0Value
   // Counter 0
   PIODA_SetCounter(wBoardNo, 0, wCounter0Config, wCounter0Value);

   // wait for Cout0 to high
   while( true ) do
   begin
      if( (PIODA_InputByte(wBase+$07) and 1) <> 0 ) then
          exit;
   end;
end;

// ******************************************************************
// Initialize the INT1 Interrupt to High,
// this will uses the Counter1 and Counter2 to trigger the interrupt.
//
// Parameters :
//   wBoardNo        :[in] Board Number, Range is 0 to 7.
//   wBase           :[in] The base-address of the board
//   wCounter1Config :[in] The configuration code of Counter1
//   wCounter1Value  :[in] 0 to &hFFFF, the value is used to set the Counter1
//                         Only the low Word (16-bits) is valid.
//   wCounter2Config :[in] The configuration code of Counter2
//   wCounter2Value  :[in] 0 to &hFFFF, the value is used to set the Counter2
//                         Only the low Word (16-bits) is valid.
// ******************************************************************
procedure INT1InitialHigh(wBoardNo:Word; wBase:LongInt;
          wCounter1Config:Word; wCounter1Value:LongInt;
          wCounter2Config:Word; wCounter2Value:LongInt);
begin
    // Cout2 as 4Mhz/( wCounter1Value * wCounter2Value)
    // Counter 1
    PIODA_SetCounter(wBoardNo, 1, wCounter1Config, wCounter1Value);
    // Counter 2
    PIODA_SetCounter(wBoardNo, 2, wCounter2Config, wCounter2Value);

    // wait for Cout2 to high
    while ( true ) do
    begin
      if( (PIODA_InputByte(wBase+$07) and 3) <> 0 ) then
          exit;
    end;
end;

end.
