unit  PIODA;          { PIODA.dll interface unit }

interface

const
  PIODA_NoError 		=0;
  PIODA_DriverOpenError	=1;
  PIODA_DriverNoOpen		=2;
  PIODA_GetDriverVersionError	=3;
  PIODA_InstallIrqError	=4;
  PIODA_ClearIntCountError    	=5;
  PIODA_GetIntCountError    	=6;
  PIODA_RegisterApcError	=7;
  PIODA_RemoveIrqError	    	=8;
  PIODA_FindBoardError        	=9;
  PIODA_ExceedBoardNumber     	=10;
  PIODA_ResetError            	=11;

  PIODA_EEPROMDataError         =12;
  PIODA_EEPROMWriteError        =13;

  // to trigger a interrupt when high -> low
  PIODA_ActiveLow               =0;
  // to trigger a interrupt when low -> high
  PIODA_ActiveHigh              =1;

  // ID
  PIO_DA                        = $800400;    // PIO-DA16/DA8/DA4


// Test functions
function  PIODA_ShortSub(nA : smallint; nB : smallint) :smallint; StdCall;
function  PIODA_FloatSub(fA :   single; fB : single) :single; StdCall;
function  PIODA_GetDllVersion : word; StdCall;

// Driver functions
function  PIODA_DriverInit : word; StdCall;
procedure PIODA_DriverClose ; StdCall;
function  PIODA_SearchCard
          (var wBoards:WORD; dwPIOPISOCardID:LongInt):WORD; StdCall;
function  PIODA_GetDriverVersion(var wDriverVer: word):WORD; StdCall;
function  PIODA_GetConfigAddressSpace
          (wBoardNo:word; var wAddrBase:LongInt;  var wIrqNo:word;
          var wSubVerdor:word; var wSubDevice:word; var wSubAux:word;
          var wSlotBus:word;   var wSlotDevice:word ): word; StdCall;
function  PIODA_ActiveBoard(wBoardNo:Word) :WORD; StdCall;
function  PIODA_WhichBoardActive :WORD; StdCall;
function  PIODA_SetCounter(wBoardNo:WORD; wWhichCounter:WORD;
          bConfig:WORD;  wValue:LongInt): WORD; StdCall;
function  PIODA_GetBaseAddress(wBoardNo:WORD):LongInt; StdCall;

// EEPROM functions
function  PIODA_EEP_READ(wBoardNo:WORD; wOffset:WORD; 
          var bHi:WORD; var bLo:WORD):WORD; StdCall;
function  PIODA_EEP_WR_EN(wBoardNo:WORD):WORD; StdCall;
function  PIODA_EEP_WR_DIS(wBoardNo:WORD):WORD; StdCall;
function  PIODA_EEP_WRITE( wBoardNo:WORD; wOffset:WORD; 
          HI:WORD; LO:WORD):WORD; StdCall;

// DA functions
//The D/A output for the PIODA_Voltage and PIODA_Current function is none calibration function
//function  PIODA_Voltage
//          (wBoardNo:WORD; wChannel:WORD; fValue:Single):WORD; StdCall;
//function  PIODA_Current
//          (wBoardNo:WORD; wChannel:WORD; fValue:Single):WORD; StdCall;
function  PIODA_CalVoltage
          (wBoardNo:WORD; wChannel:WORD; fValue:Single):WORD; StdCall;
function  PIODA_CalCurrent
          (wBoardNo:WORD; wChannel:WORD; fValue:Single):WORD; StdCall;

// DIO functions
procedure PIODA_OutputByte
          (wBaseAddress :LongInt; bOutputVal :Word); StdCall;
procedure PIODA_OutputWord
          (wBaseAddress :LongInt; wOutputVal :LongInt); StdCall;
function  PIODA_InputByte(wBaseAddress  :LongInt ) :word; StdCall;
function  PIODA_InputWord(wBaseAddress  :LongInt ) :LongInt; StdCall;
function  PIODA_DI(wBoardNo:WORD; var wVal:LongInt) :word; StdCall;
function  PIODA_DO(wBoardNo:WORD; wDO:LongInt) :word; StdCall;

// Interrupt functions
function  PIODA_IntInstall(wBoard:Word; var hEvent:LongInt; 
          wInterruptSource:Word; wActiveMode:Word):Word; StdCall;
function  PIODA_IntRemove : WORD; StdCall;
function  PIODA_IntResetCount : WORD; StdCall;
function  PIODA_IntGetCount(var dwIntCount:LongInt) : WORD; StdCall;


implementation

// Test functions
function  PIODA_ShortSub;               
          external 'PIODA.DLL' name 'PIODA_ShortSub';
function  PIODA_FloatSub;               
          external 'PIODA.DLL' name 'PIODA_FloatSub';
function  PIODA_GetDllVersion;          
          external 'PIODA.DLL' name 'PIODA_GetDllVersion';

// Driver functions
function  PIODA_DriverInit;             
          external 'PIODA.DLL' name 'PIODA_DriverInit';
procedure PIODA_DriverClose;            
          external 'PIODA.DLL' name 'PIODA_DriverClose';
function  PIODA_SearchCard;       	 
          external 'PIODA.DLL' name 'PIODA_SearchCard';
function  PIODA_GetDriverVersion; 	 
          external 'PIODA.DLL' name 'PIODA_GetDriverVersion';
function  PIODA_GetConfigAddressSpace;  
          external 'PIODA.DLL' name 'PIODA_GetConfigAddressSpace';

function  PIODA_ActiveBoard; 	         
          external 'PIODA.DLL' name 'PIODA_ActiveBoard';
function  PIODA_WhichBoardActive; 	 
          external 'PIODA.DLL' name 'PIODA_WhichBoardActive';
function  PIODA_SetCounter; 	 
          external 'PIODA.DLL' name 'PIODA_SetCounter';
function  PIODA_GetBaseAddress;
          external 'PIODA.DLL' name 'PIODA_GetBaseAddress';

// EEPROM functions
function  PIODA_EEP_READ; 	 
          external 'PIODA.DLL' name 'PIODA_EEP_READ';
function  PIODA_EEP_WR_EN; 	 
          external 'PIODA.DLL' name 'PIODA_EEP_WR_EN';
function  PIODA_EEP_WR_DIS; 	 
          external 'PIODA.DLL' name 'PIODA_EEP_WR_DIS';
function  PIODA_EEP_WRITE; 	 
          external 'PIODA.DLL' name 'PIODA_EEP_WRITE';

// DA functions
//function  PIODA_Voltage; 	 
//          external 'PIODA.DLL' name 'PIODA_Voltage';
//function  PIODA_Current; 	 
//          external 'PIODA.DLL' name 'PIODA_Current';
function  PIODA_CalVoltage; 	 
          external 'PIODA.DLL' name 'PIODA_CalVoltage';
function  PIODA_CalCurrent; 	 
          external 'PIODA.DLL' name 'PIODA_CalCurrent';

// DIO functions
procedure PIODA_OutputByte;             
          external 'PIODA.DLL' name 'PIODA_OutputByte';
procedure PIODA_OutputWord;             
          external 'PIODA.DLL' name 'PIODA_OutputWord';
function  PIODA_InputByte;              
          external 'PIODA.DLL' name 'PIODA_InputByte';
function  PIODA_InputWord;              
          external 'PIODA.DLL' name 'PIODA_InputWord';
function  PIODA_DI; 	 
          external 'PIODA.DLL' name 'PIODA_DI';
function  PIODA_DO; 	 
          external 'PIODA.DLL' name 'PIODA_DO';

// Interrupt functions
function  PIODA_IntInstall;             
          external 'PIODA.DLL' name 'PIODA_IntInstall';
function  PIODA_IntRemove;              
          external 'PIODA.DLL' name 'PIODA_IntRemove';
function  PIODA_IntGetCount;            
          external 'PIODA.DLL' name 'PIODA_IntGetCount';
function  PIODA_IntResetCount;          
          external 'PIODA.DLL' name 'PIODA_IntResetCount';

end.
