Option Strict Off
Option Explicit On
Friend Class Form1
	Inherits System.Windows.Forms.Form
	
	'/******************************************************************************/
	'/* For the proper operation the PIODA, the following functions must be used. */
	'/*    PIODA_DriverInit()     <--   initial the driver                        */
	'/*    PIODA_DriverClose()    <--   close the driver                          */
	'/******************************************************************************/
	
	Dim wBase As Integer
    Dim wTotalBoards As Integer
	Dim wInitialCode As Short
	Dim dwIntCount As Integer
	Dim hEvent As Integer
	

    Private Sub Form1_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        Dim rtn As Integer



        '********************************************************************
        '* NOTICE: call PIODA_DriverInit() to initialize the driver.        *
        '* Initial the device driver, and return the board number in the PC *
        '********************************************************************
        wInitialCode = PIODA_DriverInit()
        btnActive.Text = "Active"

        If wInitialCode <> PIODA_NoError Then

            rtn = MsgBox("Driver initialize error!!!", , "PIODA Card Error")
            btnActive.Enabled = False
            Exit Sub
        End If


        If PIODA_SearchCard(wTotalBoards, PIO_DA) <> PIODA_NoError Then

            rtn = MsgBox("Search Card Error!!", , "PIODA Card Error")
            btnActive.Enabled = False
        Else
            btnActive.Enabled = True
        End If


        eTotalBoards.Text = wTotalBoards

        nudActive.Minimum = 0
        nudActive.Maximum = wTotalBoards - 1
    End Sub
	
	Private Sub Form1_FormClosed(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
		PIODA_DriverClose()
	End Sub

	
	
	' ******************************************************************
	' Initialize the INT0 Interrupt to High
	' this will uses the Counter0 to trigger the interrupt.
	'
	' Parameters :
	'   wBoardNo        :[in] Board Number, Range is 0 to 7.
	'   wCounter0Config :[in] The configuration code of Counter0
	'   wCounter0Value  :[in] 0 to &hFFFF, the value is used to set the Counter0
	'                         Only the low Word (16-bits) is valid.
	' ******************************************************************
	Sub INT0InitialHigh(ByVal wBoardNo As Short, ByVal wBase As Integer, ByVal wCounter0Config As Short, ByVal wCounter0Value As Integer)
		' Cout0 as 4Mhz/ wCounter0Value
		' Counter 0
		PIODA_SetCounter(wBoardNo, 0, wCounter0Config, wCounter0Value)
		
		' wait for Cout0 to high
		While (True)
			If ((PIODA_InputByte(wBase + &H7s) And 1) <> 0) Then
				Exit Sub
			End If
		End While
	End Sub
	
	' ******************************************************************
	' Initialize the INT1 Interrupt to High,
	' this will uses the Counter1 and Counter2 to trigger the interrupt.
	'
	' Parameters :
	'   wBoardNo        :[in] Board Number, Range is 0 to 7.
	'   wCounter1Config :[in] The configuration code of Counter1
	'   wCounter1Value  :[in] 0 to &hFFFF, the value is used to set the Counter1
	'                         Only the low Word (16-bits) is valid.
	'   wCounter2Config :[in] The configuration code of Counter2
	'   wCounter2Value  :[in] 0 to &hFFFF, the value is used to set the Counter2
	'                         Only the low Word (16-bits) is valid.
	' ******************************************************************
	Sub INT1InitialHigh(ByVal wBoardNo As Short, ByVal wBase As Integer, ByVal wCounter1Config As Short, ByVal wCounter1Value As Integer, ByVal wCounter2Config As Short, ByVal wCounter2Value As Integer)
		' Cout2 as 4Mhz/( wCounter1Value * wCounter2Value)
		' Counter 1
		PIODA_SetCounter(wBoardNo, 1, wCounter1Config, wCounter1Value)
		'Counter 2
		PIODA_SetCounter(wBoardNo, 2, wCounter2Config, wCounter2Value)
		
		' wait for Cout2 to high
		While (True)
			If ((PIODA_InputByte(wBase + &H7s) And 3) <> 0) Then
				Exit Sub
			End If
		End While
	End Sub
	

    Private Sub btnActive_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnActive.Click
        Dim wRetVal As Short
        Dim wSelectBoard As Short

        Dim lRtn As Integer

        If btnActive.Text = "Stop" Then
            btnActive.Text = "Active"
            btnExit.Enabled = True
            Exit Sub
        End If

        List1.Items.Clear()

        wSelectBoard = nudActive.Value
        If wSelectBoard > Val(eTotalBoards.Text) - 1 Or wSelectBoard < 0 Then
            List1.Items.Add("Invalid board number, Pls retry!!")
            Exit Sub
        End If

        wBase = PIODA_GetBaseAddress(wSelectBoard)
        If wBase = 0 Then
            List1.Items.Add("Get Base-Address Error !!")
            Exit Sub
        End If

        PIODA_IntResetCount()

        dwIntCount = 0
        hEvent = CreateEvent(0, False, False, 0)
        If hEvent = 0 Then
            List1.Items.Add("Create Event Error!!")
            Exit Sub
        Else
            List1.Items.Add("Create Event OK!!")
        End If

        INT0InitialHigh(wSelectBoard, wBase, &H36S, 20000)
        'INT1InitialHigh  wSelectBoard, wBase, &h76, 400, &hb6, 100

        '// Initial High and Active Low
        '// The Interrupt-Source can be:
        '//         0: INT0
        '//         1: INT1
        wRetVal = PIODA_IntInstall(wSelectBoard, hEvent, 0, PIODA_ActiveLow)
        If wRetVal <> PIODA_NoError Then
            List1.Items.Add("Interrupt Install Error!!")
            List1.Items.Add("Close Event")
            CloseHandle(hEvent)
            Exit Sub
        Else
            List1.Items.Add("Interrupt Install OK!!")
        End If

        btnActive.Text = "Stop"
        btnExit.Enabled = False
        List1.Items.Add("  ")
        List1.Items.Add("Click the button 'Stop' to stop!")
        System.Windows.Forms.Application.DoEvents()

        Dim j As Short
        j = 0
        Do While (True)
            List1.Items.Add("(#" + CStr(j) + ")   Wait Event .....")
            System.Windows.Forms.Application.DoEvents()
            lRtn = WaitForSingleObject(hEvent, INFINITE)
            List1.Items.Add("(#" + CStr(j) + ")   Got Event !!")
            System.Windows.Forms.Application.DoEvents()
            '***** Do some thing in here ****
            '   :
            '   :
            '   :
            List1.Items.Add("  ")
            List1.SelectedIndex = List1.Items.Count - 1
            System.Windows.Forms.Application.DoEvents()
            If btnActive.Text = "Active" Then
                Exit Do
            End If
            j = j + 1
        Loop

        List1.Items.Add(" ")

        '// Revmove the Interrupt
        PIODA_IntRemove()
        List1.Items.Add("Stop Interrupt!!")
        CloseHandle(hEvent)

        '// How many times the interrupt occur
        wRetVal = PIODA_IntGetCount(dwIntCount)
        If wRetVal <> PIODA_NoError Then
            List1.Items.Add("Get Interrupt Counter Error!!")
        Else
            List1.Items.Add("Interrupt Counter:" & Str(dwIntCount))
        End If

        List1.Items.Add("end")
    End Sub

    Private Sub btnExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExit.Click
        End
    End Sub
End Class