/****************************************************************************/
/*  Digital Input/Output DEMO for PIO_DA card:                              */
/****************************************************************************/
/* For the proper operation purpose, the following functions must be used.  */
/*    PIODA_DriverInit();     <--   initial the driver                    */
/*    PIODA_DriverClose();    <--   close the driver                      */
/****************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include "PIODA.H"
#include "resource.h"

WORD         wTotalBoard,wInitialCode;
HINSTANCE    hInst;

LRESULT CALLBACK  WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL    CALLBACK  DialogFunc(HWND, UINT, WPARAM, LPARAM);

DWORD    wGetAddrBase;   // base address assigned for board
WORD     wGetIrqNo;      // IRQ Level assigned for board
WORD     wGetSubVendor,wGetSubDevice,wGetSubAux,wGetSlotBus,wGetSlotDevice;
/* -------------------------------------------------------------------- */
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "PIO-DA4/DA8/DA16 Digital In/Out Demo";
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof(wndclass);
     wndclass.style         = CS_HREDRAW|CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc;
     wndclass.cbClsExtra    = 0;
     wndclass.cbWndExtra    = 0;
     wndclass.hInstance     = hInstance;
     wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
     wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
     wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
     wndclass.lpszMenuName  = "MYMENU";
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

     hInst=hInstance;
     RegisterClassEx(&wndclass) ;
     hwnd=CreateWindow(szAppName,"PIO-DA4/DA8/DA16 Digital In/Out Demo:",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;
     ShowWindow(hwnd,iCmdShow);
     UpdateWindow(hwnd);

     while (GetMessage(&msg, NULL, 0, 0))
     {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
     }
     return msg.wParam;
     }

/* -------------------------------------------------------------------- */

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
   static int  cxChar, cyChar, cxClient, cyClient, cxBuffer, cyBuffer,
               xCaret, yCaret;
   static char cBuf[80];
   int         response;

  switch (iMsg)
  {
     case WM_CREATE :  // window initial
       /***************************************************************/
       /* NOTICE: call PIODA_DriverInit() to initialize the driver. */
       /***************************************************************/
       // Initial the device driver, and return the board number in the PC
       wInitialCode=PIODA_DriverInit();
       if( wInitialCode!=PIODA_NoError )
       {
          MessageBox(hwnd,"No PIO-DA4/DA8/DA16 card in this system !!!","PIO Card Error",MB_OK);
       }

       /*****************************************************************/
       /*  Searching the PIO-DA16/DA8/DA4 that installed in the system  */
       /*****************************************************************/
       PIODA_SearchCard(&wTotalBoard,PIO_DA);
       if( wTotalBoard==0 )
       {
            Beep(100,100);
            MessageBox(hwnd,"No PIO-DA4/DA8/DA16 card in this system !!!","PIO Card Error",MB_OK);
            PostQuitMessage(0);
       }
       return 0;
     case WM_COMMAND:
       switch( LOWORD(wParam) )
       {
          case  ID_DEMO:
             DialogBox(hInst,"MYDB",hwnd,DialogFunc);
             break;
          case  ID_EXIT:
             Beep(100,100);
             response=MessageBox(hwnd,"Are you sure to quit?","Quit DEMO",MB_YESNO);
             if( response==IDYES )
                PostQuitMessage(0);
             break;
       }
       break;
     case WM_DESTROY :
       /**************************************************************/
       /* NOTICE: call PIODA_DriverClose() to close the driver.    */
       /**************************************************************/
       PIODA_DriverClose();         // close the driver
       PostQuitMessage(0);
       return 0 ;
  }
  return DefWindowProc(hwnd, iMsg, wParam, lParam);
}


/* -------------------------------------------------------------------- */
BOOL  CALLBACK  DialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   WORD     wIndex,wRetVal;
   char     str[80],*stopstring;
   long     i;
   WORD     wBoardToActive=0;
   WORD     wDigitalOut;
   BYTE     bDigitalIn;

   switch(message)
   {
      case  WM_INITDIALOG:
         sprintf(str,"%d",wTotalBoard);   // initialize list box
         SetDlgItemText(hdwnd,IDC_EDITBOARDS,str);
         for(wIndex=0;  wIndex<wTotalBoard;  wIndex++)
         {
            sprintf(str,"Board %02d",wIndex);
            SendDlgItemMessage(hdwnd,IDC_LIST1,LB_ADDSTRING,0,(LPARAM)str);
         }
         // select first item
         SendDlgItemMessage(hdwnd,IDC_LIST1,LB_SETCURSEL,0,0);

         // get the first board information and display  it
         wRetVal=PIODA_GetConfigAddressSpace(0, &wGetAddrBase, &wGetIrqNo,
                &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
         if( wRetVal==PIODA_NoError )
         {
            sprintf(str,"%04x",wGetAddrBase);
            SetDlgItemText(hdwnd,IDC_EDITBASE,str);
            sprintf(str,"%x",wGetIrqNo);
            SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
         }

         // Initialize digital output value
         wDigitalOut=0x55;
         sprintf(str,"%02x",wDigitalOut);
         SetDlgItemText(hdwnd,IDC_EDITDO0,str);
         wDigitalOut=0x55;
         sprintf(str,"%02x",wDigitalOut);
         SetDlgItemText(hdwnd,IDC_EDITDO1,str);

         // Initialize digital input value
         sprintf(str,"   ");
         SetDlgItemText(hdwnd,IDC_EDITDI0,str);
         sprintf(str,"   ");
         SetDlgItemText(hdwnd,IDC_EDITDI1,str);

         return 1;
      case  WM_COMMAND:

         switch( LOWORD(wParam) )
         {
            case  IDC_LIST1:  // select a board to active
                if( HIWORD(wParam)==LBN_SELCHANGE )
                {
                  i=SendDlgItemMessage(hdwnd,IDC_LIST1,LB_GETCURSEL,0,0);

                  // set the board i to active
                  wBoardToActive=(WORD)i;

                  // display configuration for board i
                  wRetVal=PIODA_GetConfigAddressSpace(wBoardToActive, &wGetAddrBase, &wGetIrqNo,
                        &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
                  if( wRetVal==PIODA_NoError )
                  {
                     sprintf(str,"%04x",wGetAddrBase);
                     SetDlgItemText(hdwnd,IDC_EDITBASE,str);
                     sprintf(str,"%x",wGetIrqNo);
                     SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
                  }

                }
                return 1;
            case  IDC_BUTTONDO0: // DigitalOut button is pressed
                GetDlgItemText(hdwnd,IDC_EDITDO0,str,80);
                wDigitalOut=(WORD)strtoul(str,&stopstring,16);
                // output DO_00 to DO_07
                PIODA_OutputByte(wGetAddrBase+0xf8,(BYTE)wDigitalOut);
                return 1;
            case  IDC_BUTTONDO1: // DigitalOut button is pressed
                GetDlgItemText(hdwnd,IDC_EDITDO1,str,80);
                wDigitalOut=(WORD)strtoul(str,&stopstring,16);
                // output DO_08 to DO_015
                PIODA_OutputByte(wGetAddrBase+0xfc,(BYTE)wDigitalOut);
               return 1;

            case  IDC_BUTTONDI0: // DigitalIn button is pressed
                // input DI_00 to DI_07
                bDigitalIn=(BYTE)PIODA_InputByte(wGetAddrBase+0xe0);
                sprintf(str,"%02x",bDigitalIn);
                SetDlgItemText(hdwnd,IDC_EDITDI0,str);
                return 1;
            case  IDC_BUTTONDI1: // DigitalIn button is pressed
                // input DI_08 to DI_15
                bDigitalIn=(BYTE)PIODA_InputByte(wGetAddrBase+0xe4);
                sprintf(str,"%02x",bDigitalIn);
                SetDlgItemText(hdwnd,IDC_EDITDI1,str);
                return 1;

            case  IDCANCEL:
                EndDialog(hdwnd,0);
                return 1;
         }
   }
   return 0;
}
