/****************************************************************************/
/*  D/A output DEMO *WITH* calibration for PIO-DA16/8/4 card:               */
/****************************************************************************/
/* For the proper operation purpose, the following functions must be used.  */
/*    PIODA_DriverInit();     <--   initial the driver                      */
/*    PIODA_DriverClose();    <--   close the driver                        */
/****************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include "PIODA.H"
#include "resource.h"

WORD         wTotalBoard,wInitialCode;
HINSTANCE    hInst;

DWORD        wGetAddrBase;   // base address assigned for board
WORD         wGetIrqNo;      // IRQ Level assigned for board
WORD         wGetSubVendor,wGetSubDevice,wGetSubAux,wGetSlotBus,wGetSlotDevice;

//--------- function propotype ----------------------------
LRESULT   CALLBACK  WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL      CALLBACK  DialogFunc(HWND, UINT, WPARAM, LPARAM);


/* -------------------------------------------------------------------- */
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "PIO-DA4/DA8/DA16 Digital In/Out Demo";
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof(wndclass);
     wndclass.style         = CS_HREDRAW|CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc;
     wndclass.cbClsExtra    = 0;
     wndclass.cbWndExtra    = 0;
     wndclass.hInstance     = hInstance;
     wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
     wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
     wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
     wndclass.lpszMenuName  = "MYMENU";
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

     hInst=hInstance;
     RegisterClassEx(&wndclass) ;
     hwnd=CreateWindow(szAppName,"PIO-DA4/DA8/DA16 Digital In/Out Demo:",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;
     ShowWindow(hwnd,iCmdShow);
     UpdateWindow(hwnd);

     while (GetMessage(&msg, NULL, 0, 0))
     {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
     }
     return msg.wParam;
     }

/* -------------------------------------------------------------------- */

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
   static int  cxChar, cyChar, cxClient, cyClient, cxBuffer, cyBuffer,
               xCaret, yCaret;
   static char cBuf[80];
   int         response;

  switch (iMsg)
  {
     case WM_CREATE :  // window initial
       /***************************************************************/
       /* NOTICE: call PIODA_DriverInit() to initialize the driver. */
       /***************************************************************/
       // Initial the device driver, and return the board number in the PC
       wInitialCode=PIODA_DriverInit();
       if( wInitialCode!=PIODA_NoError )
       {
          MessageBox(hwnd,"No PIO-DA4/DA8/DA16 card in this system !!!","PIO Card Error",MB_OK);
       }

       /*****************************************************************/
       /*  Searching the PIO-DA16/DA8/DA4 that installed in the system  */
       /*****************************************************************/
       PIODA_SearchCard(&wTotalBoard,PIO_DA);
       if( wTotalBoard==0 )
       {
            Beep(100,100);
            MessageBox(hwnd,"No PIO-DA4/DA8/DA16 card in this system !!!","PIO Card Error",MB_OK);
            PostQuitMessage(0);
       }
       return 0;
     case WM_COMMAND:
       switch( LOWORD(wParam) )
       {
          case  ID_DEMO:
             DialogBox(hInst,"MYDB",hwnd,DialogFunc);
             break;
          case  ID_EXIT:
             Beep(100,100);
             response=MessageBox(hwnd,"Are you sure to quit?","Quit DEMO",MB_YESNO);
             if( response==IDYES )
                PostQuitMessage(0);
             break;
       }
       break;
     case WM_DESTROY :
       /**************************************************************/
       /* NOTICE: call PIODA_DriverClose() to close the driver.    */
       /**************************************************************/
       PIODA_DriverClose();         // close the driver
       PostQuitMessage(0);
       return 0 ;
  }
  return DefWindowProc(hwnd, iMsg, wParam, lParam);
}


WORD     wChannelToActive;
/* -------------------------------------------------------------------- */
BOOL  CALLBACK  DialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   char     str[80];  // ,*stopstring;
   long     i;
   int      iResponse;
   WORD     wBoardToActive=0;
   float    fVoltOut,fCurrentOut;
   WORD     wIndex,wRetVal;

   switch(message)
   {
      case  WM_INITDIALOG:

         //--- initialize Voltage Channel list ---
         for(wIndex=0;  wIndex<16;  wIndex++)
         {
            sprintf(str,"CH:%02d",wIndex);
            SendDlgItemMessage(hdwnd,IDC_CHANNELVOLT,LB_ADDSTRING,0,(LPARAM)str);
         }
         // select first item
         SendDlgItemMessage(hdwnd,IDC_CHANNELVOLT,LB_SETCURSEL,0,0);
         wChannelToActive=0;

         //--- initialize Current Channel list ---
         for(wIndex=0;  wIndex<16;  wIndex++)
         {
            sprintf(str,"CH:%02d",wIndex);
            SendDlgItemMessage(hdwnd,IDC_CHANNELCURRENT,LB_ADDSTRING,0,(LPARAM)str);
         }
         // select first item
         SendDlgItemMessage(hdwnd,IDC_CHANNELCURRENT,LB_SETCURSEL,0,0);
         wChannelToActive=0;

         sprintf(str,"%d",wTotalBoard);   // initialize list box
         SetDlgItemText(hdwnd,IDC_EDITBOARDS,str);
         for(wIndex=0;  wIndex<wTotalBoard;  wIndex++)
         {
            sprintf(str,"Board %02d",wIndex);
            SendDlgItemMessage(hdwnd,IDC_LIST1,LB_ADDSTRING,0,(LPARAM)str);
         }
         // select first item
         SendDlgItemMessage(hdwnd,IDC_LIST1,LB_SETCURSEL,0,0);

         // get the first board information and display  it
         wRetVal=PIODA_GetConfigAddressSpace(0, &wGetAddrBase, &wGetIrqNo,
                &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
         if( wRetVal==PIODA_NoError )
         {
            sprintf(str,"%04x",wGetAddrBase);
            SetDlgItemText(hdwnd,IDC_EDITBASE,str);
            sprintf(str,"%x",wGetIrqNo);
            SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
         }


         // Initialize voltage output value as 1.23V
         fVoltOut=1.23F; // float data type
         sprintf(str,"%05.2f",fVoltOut);
         SetDlgItemText(hdwnd,IDC_EDITVOLT,str);

         // Initialize current output value as 1.23mA
         fCurrentOut=1.23F;  // float data type
         sprintf(str,"%05.2f",fCurrentOut);
         SetDlgItemText(hdwnd,IDC_EDITCURRENT,str);
         return 1;
      case  WM_COMMAND:

         switch( LOWORD(wParam) )
         {
            case  IDC_LIST1:  // select a board to active
                if( HIWORD(wParam)==LBN_SELCHANGE )
                {
                  i=SendDlgItemMessage(hdwnd,IDC_LIST1,LB_GETCURSEL,0,0);

                  // set the board i to active
                  wBoardToActive=(WORD)i;

                  // display configuration for board i
                  wRetVal=PIODA_GetConfigAddressSpace(wBoardToActive, &wGetAddrBase, &wGetIrqNo,
                        &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
                  if( wRetVal==PIODA_NoError )
                  {
                     sprintf(str,"%04x",wGetAddrBase);
                     SetDlgItemText(hdwnd,IDC_EDITBASE,str);
                     sprintf(str,"%x",wGetIrqNo);
                     SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
                  }


                }
                return 1;
            case  IDC_CHANNELVOLT:  // select a channel to output volt
                // Notice: Don't Exceed the channel No. for a PIO-DA board support
                if( HIWORD(wParam)==LBN_SELCHANGE )
                {
                  i=SendDlgItemMessage(hdwnd,IDC_CHANNELVOLT,LB_GETCURSEL,0,0);

                  // set the channel for voltage output
                  wChannelToActive=(WORD)i;
                  if( wChannelToActive>7 )
                  {
                      Beep(200,200);
                      sprintf(str,"The PIO-DA16 has been plugged in ?");
                      iResponse=MessageBox(hdwnd,str,"Please make sure:",MB_YESNO+MB_ICONEXCLAMATION);
                      if( iResponse==IDNO)
                      {
                         SendDlgItemMessage(hdwnd,IDC_CHANNELVOLT,LB_SETCURSEL,0,0);
                         wChannelToActive=0;
                      }
                  }
                  else if( wChannelToActive>3 && wChannelToActive<=7)
                  {
                      Beep(200,200);
                      sprintf(str,"The PIO-DA8 or PIO-DA16 has been plugged in ?");
                      iResponse=MessageBox(hdwnd,str,"Please make sure:",MB_YESNO+MB_ICONEXCLAMATION);
                      if( iResponse==IDNO)
                      {
                         SendDlgItemMessage(hdwnd,IDC_CHANNELVOLT,LB_SETCURSEL,0,0);
                         wChannelToActive=0;
                      }

                  }
                  else
                  {
                  }
                }
                return 1;
            case  IDC_CHANNELCURRENT:  // select a channel to output current
                // Notice: Don't Exceed the channel No. for a PIO-DA board support
                if( HIWORD(wParam)==LBN_SELCHANGE )
                {
                  i=SendDlgItemMessage(hdwnd,IDC_CHANNELCURRENT,LB_GETCURSEL,0,0);

                  // set the channel for current output
                  wChannelToActive=(WORD)i;
                  if( wChannelToActive>7 )
                  {
                      Beep(200,200);
                      sprintf(str,"The PIO-DA16 has been plugged in ?");
                      iResponse=MessageBox(hdwnd,str,"Please make sure:",MB_YESNO+MB_ICONEXCLAMATION);
                      if( iResponse==IDNO)
                      {
                         SendDlgItemMessage(hdwnd,IDC_CHANNELCURRENT,LB_SETCURSEL,0,0);
                         wChannelToActive=0;
                      }
                  }
                  else if( wChannelToActive>3 && wChannelToActive<=7)
                  {
                      Beep(200,200);
                      sprintf(str,"The PIO-DA8 or PIO-DA16 has been plugged in ?");
                      iResponse=MessageBox(hdwnd,str,"Please make sure:",MB_YESNO+MB_ICONEXCLAMATION);
                      if( iResponse==IDNO)
                      {
                         SendDlgItemMessage(hdwnd,IDC_CHANNELCURRENT,LB_SETCURSEL,0,0);
                         wChannelToActive=0;
                      }
                  }
                  else
                  {
                  }
                }
                return 1;
            case  IDC_BUTTONDO0: // Voltage Out button is pressed
                GetDlgItemText(hdwnd,IDC_EDITVOLT,str,80);
                fVoltOut=(float)atof(str);
                //-----  output voltage ----------
                PIODA_CalVoltage(wBoardToActive,wChannelToActive,fVoltOut);
                return 1;
            case  IDC_BUTTONDO1: // Voltage Out button is pressed
                GetDlgItemText(hdwnd,IDC_EDITCURRENT,str,80);
                fCurrentOut=(float)atof(str);
                //-----  output current ----------
                PIODA_CalCurrent(wBoardToActive,wChannelToActive,fCurrentOut);
               return 1;
            case  IDCANCEL:
                EndDialog(hdwnd,0);
                return 1;
         }
   }
   return 0;
}
