/****************************************************************************/
/*  PIO-DA4/8/16:   Interrupt DEMO for INT_CHAN_0 & INT_CHAN_1              */
/****************************************************************************/
/* Notice: because we programmimg the Cout0 & Cout2 to *High*,              */
/*         So, we wish the interrupt is triggered when the                  */
/*         interrupt source bit(Cout0, Cout2) change from HIGH to LOW       */
/*  That is: Initial HIGH and Active LOW                                    */
/****************************************************************************/
/* For the proper operation the PIO-DA, the following functions must be used*/
/*    PIODA_DriverInit();     <--   initial the driver                      */
/*    PIODA_DriverClose();    <--   close the driver                        */
/****************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include "PIODA.H"
#include "resource.h"

WORD         wTotalBoard,wInitialCode;
HINSTANCE    hInst;

HANDLE       hEvent;
DWORD        Tid;
WORD         dwServiceCallCount=0;

DWORD    wGetAddrBase;   // the base address assigned for board
WORD     wGetIrqNo;      // the IRQ Level assigned for board
WORD     wGetSubVendor,wGetSubDevice,wGetSubAux,wGetSlotBus,wGetSlotDevice;


LRESULT CALLBACK    WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL    CALLBACK    DialogFunc(HWND, UINT, WPARAM, LPARAM);

void InitialINTChannel0(WORD wBoardNo, DWORD wAddrBase);
void InitialINTChannel1(WORD wBoardNo, DWORD wAddrBase);

static char szAppName[] = "PIO-DA Interrupt Demo";
static char szWindowTitle[]="PIO-DA Interrupt Demo";


/* -------------------------------------------------------------------- */
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof(wndclass);
     wndclass.style         = CS_HREDRAW|CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc;
     wndclass.cbClsExtra    = 0;
     wndclass.cbWndExtra    = 0;
     wndclass.hInstance     = hInstance;
     wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
     wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
     wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
     wndclass.lpszMenuName  = "MYMENU";
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

     hInst=hInstance;
     RegisterClassEx(&wndclass) ;
     hwnd=CreateWindow(szAppName,szWindowTitle,
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;
     ShowWindow(hwnd,iCmdShow);
     UpdateWindow(hwnd);

     while (GetMessage(&msg, NULL, 0, 0))
     {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
     }
     return msg.wParam;
     }

/* -------------------------------------------------------------------- */

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
   static int  cxChar, cyChar, cxClient, cyClient, cxBuffer, cyBuffer,
               xCaret, yCaret;
   static char cBuf[80];
   int         response;

  switch (iMsg)
  {
     case WM_CREATE :  // window initial
       /***************************************************************/
       /* NOTICE: call PIODA_DriverInit() to initialize the driver.   */
       /***************************************************************/
       // Initial the device driver, and return the board number in the PC
       wInitialCode=PIODA_DriverInit();
       if( wInitialCode!=PIODA_NoError )
       {
          MessageBox(hwnd,"No PIO-DA card in this system !!!","PIO-DA Card Error",MB_OK);
       }

       // Searching how many PIO-DA card in PC
       PIODA_SearchCard(&wTotalBoard,PIO_DA);
       if( wTotalBoard==0 )
       {
            Beep(100,100);
            MessageBox(hwnd,"No PIO-DA card in this system !!!","PIO-DA Card Error",MB_OK);
            PostQuitMessage(0);
       }
       return 0;
     case WM_COMMAND:
       switch( LOWORD(wParam) )
       {
          case  ID_DEMO:
             DialogBox(hInst,"MYDB",hwnd,DialogFunc);
             break;
          case  ID_EXIT:
             Beep(100,100);
             response=MessageBox(hwnd,"Are you sure to quit?","Quit DEMO",MB_YESNO);
             if( response==IDYES )
                PostQuitMessage(0);
             break;
       }
       break;
     case WM_DESTROY :
       /**************************************************************/
       /* NOTICE: call PIODA_DriverClose() to close the driver.      */
       /**************************************************************/
       PIODA_DriverClose();         // close the driver
       PostQuitMessage(0);
       return 0 ;
  }
  return DefWindowProc(hwnd, iMsg, wParam, lParam);
}

/* -------------------------------------------------------------------- */
BOOL  CALLBACK  DialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   WORD            wIndex,wRetVal;
   char            str[80];  //,*stopstring;
   long            i;
   DWORD           dwIntCount;
   MSG             msg;
   WORD            wInterruptLevel;
   static WORD     wBoardToActive=0;
   static WORD     wInterruptSource;
   static WORD     wInPortToActive,wOutPortToActive;

   switch(message)
   {
      case  WM_INITDIALOG:
         // the first interrupt source as default
         SendDlgItemMessage(hdwnd, IDC_RADIO1, BM_SETCHECK, 1, 0);

         // Because the Cout0 & Cout2 is initialized to HIGH
         // so set the Interrupt Active LOW as deult
         SendDlgItemMessage(hdwnd, IDC_RADIOLEVEL1, BM_SETCHECK, 1, 0);

         sprintf(str,"%d",wTotalBoard);   // initialize list box
         SetDlgItemText(hdwnd,IDC_EDITBOARDS,str);
         for(wIndex=0;  wIndex<wTotalBoard;  wIndex++)
         {
            sprintf(str,"Board %02d",wIndex);
            SendDlgItemMessage(hdwnd,IDC_LIST1,LB_ADDSTRING,0,(LPARAM)str);
         }
         // select first item
         SendDlgItemMessage(hdwnd,IDC_LIST1,LB_SETCURSEL,0,0);
         wBoardToActive=0;

         // get the first board information and display  it
         wRetVal=PIODA_GetConfigAddressSpace(0, &wGetAddrBase, &wGetIrqNo,
                &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
         if( wRetVal==PIODA_NoError )
         {
            sprintf(str,"%04x",wGetAddrBase);
            SetDlgItemText(hdwnd,IDC_EDITBASE,str);
            sprintf(str,"%x",wGetIrqNo);
            SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
         }
         return 1;
      case  WM_COMMAND:
         switch( LOWORD(wParam) )
         {
            case  IDC_LIST1:  // select a board to active
                if( HIWORD(wParam)==LBN_SELCHANGE )
                {
                  i=SendDlgItemMessage(hdwnd,IDC_LIST1,LB_GETCURSEL,0,0);

                  // set the board i to active
                  wBoardToActive=(WORD)i;

                  // display configuration for board i
                  wRetVal=PIODA_GetConfigAddressSpace(wBoardToActive, &wGetAddrBase, &wGetIrqNo,
                        &wGetSubVendor,&wGetSubDevice,&wGetSubAux,&wGetSlotBus,&wGetSlotDevice);
                  if( wRetVal==PIODA_NoError )
                  {
                     sprintf(str,"%04x",wGetAddrBase);
                     SetDlgItemText(hdwnd,IDC_EDITBASE,str);
                     sprintf(str,"%x",wGetIrqNo);
                     SetDlgItemText(hdwnd,IDC_EDITIRQ,str);
                  }
                }
                return 1;
            case  IDC_INTERRUPT:
                  //--- deciding the interrupt active level ---
                  if( SendDlgItemMessage(hdwnd, IDC_RADIOLEVEL0, BM_GETCHECK, 0, 0) )
                       wInterruptLevel=1;
                  else
                       wInterruptLevel=0;

                  //--- deciding the interrupt source we selected ---
                  if( SendDlgItemMessage(hdwnd, IDC_RADIO1, BM_GETCHECK, 0, 0) )
                  {
                       wInterruptSource=0;
                       InitialINTChannel0(wBoardToActive, wGetAddrBase);
                  }
                  else
                  {
                       wInterruptSource=1;
                       InitialINTChannel1(wBoardToActive, wGetAddrBase);
                  }
                  wRetVal=PIODA_IntResetCount();
                  if( wRetVal>0 )
                  {
                      EndDialog(hdwnd,0);
                      return 1;
                  }
//MessageBox(hdwnd,"2 !!!","PIO-DA Card Error",MB_OK);
                  hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
                  //--- initial all counter to 0 ---
                  dwIntCount=0;
                  dwServiceCallCount=0;
                  sprintf(str,"%ld",dwIntCount);
                  SetDlgItemText(hdwnd,IDC_EDITINTERRUPT,str);
                  sprintf(str,"%ld",dwServiceCallCount);
                  SetDlgItemText(hdwnd,IDC_EDITSERVICE,str);
//MessageBox(hdwnd,"3 !!!","PIO-DA Card Error",MB_OK);
                  /*************************************************************/
                  /*  Initial HIGH and Active LOW                              */
                  /*************************************************************/
                  wRetVal=PIODA_IntInstall(wBoardToActive, &hEvent, wInterruptSource, wInterruptLevel);
                  if( wRetVal>0 )
                  {
                      sprintf(str,"Error code: %03d",wRetVal);
                      MessageBox(hdwnd,str,"PIODA_IntInstall() Error !!!",MB_OK);
                      EndDialog(hdwnd,0);
                      return 1;
                  }

//MessageBox(hdwnd,"4 !!!","PIO-DA Card Error",MB_OK);
                  while (TRUE)
                  {
                     if ( WaitForSingleObject(hEvent, 100) == WAIT_OBJECT_0 )  // wait for 100 milliSecond
                     {
                     	//***** Do some thing in here ****
                     	//   :
                     	//   :
                     	//   :
                     	dwServiceCallCount++;
                     }
                     //--- press right mouse button to quit ---
                     if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
                     {
                        if( msg.message==WM_RBUTTONDOWN )
                              break;
                     }
                  }
                  PIODA_IntRemove( );
                  CloseHandle(hEvent);

                  //*** How many times the interrupt occur ***
                  wRetVal=PIODA_IntGetCount(&dwIntCount);
                  if( wRetVal>PIODA_NoError )
                  {
                     MessageBox(hdwnd,"PIODA_IntGetCount() Error !!!","Function Error",MB_OK);
                     return 1;
                  }

                  //*** How many times the interrupt occurs ***
                  sprintf(str,"%ld",dwIntCount);
                  SetDlgItemText(hdwnd,IDC_EDITINTERRUPT,str);
                  //*** How many times the hEvent is signaled ***
                  sprintf(str,"%ld",dwServiceCallCount);
                  SetDlgItemText(hdwnd,IDC_EDITSERVICE,str);
                  return  1;
            case  IDCANCEL:
                EndDialog(hdwnd,0);
                return 1;
         }
   }
   return 0;
}

void InitialINTChannel0(WORD wBoardNo, DWORD wAddrBase)
{
   // To program Cout0 as 200Hz        // CLK source is 4Mhz
   PIODA_SetCounter(wBoardNo, 0,0x36,0x4e20); // Counter0, Mode3, 4M/(0x4e20)

   // wait for Cout0 to high
   for( ;  ;  )
   {
      if( (PIODA_InputByte(wAddrBase+0x07)&1) )
          break;
   }
}

void InitialINTChannel1(WORD wBoardNo, DWORD wAddrBase)
{
   // To program Cout2 as 100Hz                  // CLK source is 4Mhz
   PIODA_SetCounter(wBoardNo, 1,0x76,0x0190); // Counter1, Mode3, div 400
   PIODA_SetCounter(wBoardNo, 2,0xb6,0x0046); // Counter2, Mode3, div 100
                                                 // Cout2 as 4Mhz/(400*100)

   // wait for Cout2 to high
   for( ;  ;  )
   {
      if( (PIODA_InputByte(wAddrBase+0x07)&3) )
          break;
   }
}

